@@ -20,7 +20,7 @@ import (
 	"os"
 	"path/filepath"
 
-	"github.com/coreos/ignition/v2/config/v3_5_experimental/types"
+	"github.com/coreos/ignition/v2/config/v3_6_experimental/types"
 	"github.com/coreos/ignition/v2/internal/distro"
 	"github.com/coreos/ignition/v2/internal/exec/stages"
 	"github.com/coreos/ignition/v2/internal/exec/util"
@@ -164,7 +164,7 @@ func (s *stage) relabel(paths ...string) {
 // relabelFiles relabels all the files that were marked for relabeling using
 // the libselinux APIs.
 func (s *stage) relabelFiles() error {
-	if s.toRelabel == nil || len(s.toRelabel) == 0 {
+	if len(s.toRelabel) == 0 {
 		return nil
 	}
 
