@@ -39,6 +39,8 @@ import (
 	"golang.org/x/oauth2/google"
 	"google.golang.org/api/option"
 
+	"github.com/Azure/azure-sdk-for-go/sdk/azidentity"
+	"github.com/Azure/azure-sdk-for-go/sdk/storage/azblob"
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/arn"
 	"github.com/aws/aws-sdk-go/aws/awserr"
@@ -62,7 +64,7 @@ var (
 	// config is being fetched
 	configHeaders = http.Header{
 		"Accept-Encoding": []string{"identity"},
-		"Accept":          []string{"application/vnd.coreos.ignition+json;version=3.4.0, */*;q=0.1"},
+		"Accept":          []string{"application/vnd.coreos.ignition+json;version=3.5.0, */*;q=0.1"},
 	}
 
 	// We could derive this info from aws-sdk-go/aws/endpoints/defaults.go,
@@ -98,6 +100,10 @@ type Fetcher struct {
 	// It is used when fetching resources from GCS.
 	GCSSession *storage.Client
 
+	// Azure credential to use when fetching resources from Azure Blob Storage.
+	// using DefaultAzureCredential()
+	AzSession *azidentity.DefaultAzureCredential
+
 	// Whether to only attempt fetches which can be performed offline. This
 	// currently only includes the "data" scheme. Other schemes will result in
 	// ErrNeedNet. In the future, we can improve on this by dropping this
@@ -149,7 +155,17 @@ func (f *Fetcher) FetchToBuffer(u url.URL, opts FetchOptions) ([]byte, error) {
 	dest := new(bytes.Buffer)
 	switch u.Scheme {
 	case "http", "https":
-		err = f.fetchFromHTTP(u, dest, opts)
+		isAzureBlob := strings.HasSuffix(u.Host, ".blob.core.windows.net")
+		if f.AzSession != nil && isAzureBlob {
+			err = f.fetchFromAzureBlob(u, dest, opts)
+			if err != nil {
+				f.Logger.Info("could not fetch %s via Azure credentials: %v", u.String(), err)
+				f.Logger.Info("falling back to HTTP fetch")
+			}
+		}
+		if !isAzureBlob || f.AzSession == nil || err != nil {
+			err = f.fetchFromHTTP(u, dest, opts)
+		}
 	case "tftp":
 		err = f.fetchFromTFTP(u, dest, opts)
 	case "data":
@@ -207,10 +223,21 @@ func (f *Fetcher) Fetch(u url.URL, dest *os.File, opts FetchOptions) error {
 	if f.Offline && util.UrlNeedsNet(u) {
 		return ErrNeedNet
 	}
-
+	var err error
 	switch u.Scheme {
 	case "http", "https":
-		return f.fetchFromHTTP(u, dest, opts)
+		isAzureBlob := strings.HasSuffix(u.Host, ".blob.core.windows.net")
+		if f.AzSession != nil && isAzureBlob {
+			err = f.fetchFromAzureBlob(u, dest, opts)
+			if err != nil {
+				f.Logger.Info("could not fetch %s via Azure credentials: %v", u.String(), err)
+				f.Logger.Info("falling back to HTTP fetch")
+			}
+		}
+		if !isAzureBlob || f.AzSession == nil || err != nil {
+			err = f.fetchFromHTTP(u, dest, opts)
+		}
+		return err
 	case "tftp":
 		return f.fetchFromTFTP(u, dest, opts)
 	case "data":
@@ -386,8 +413,8 @@ func (f *Fetcher) fetchFromGCS(u url.URL, dest io.Writer, opts FetchOptions) err
 		if metadata.OnGCE() {
 			// check whether the VM is associated with a service
 			// account
-			if _, err := metadata.Scopes(""); err == nil {
-				id, _ := metadata.ProjectID()
+			if _, err := metadata.ScopesWithContext(ctx, ""); err == nil {
+				id, _ := metadata.ProjectIDWithContext(ctx)
 				creds := &google.Credentials{
 					ProjectID:   id,
 					TokenSource: google.ComputeTokenSource("", storage.ScopeReadOnly),
@@ -554,6 +581,54 @@ func (f *Fetcher) fetchFromS3WithCreds(ctx context.Context, dest s3target, input
 	return nil
 }
 
+// parse the a Azure Blob Storage URL into its components:
+// storage account, container, and file
+func (f *Fetcher) parseAzureStorageUrl(u url.URL) (string, string, string, error) {
+	storageAccount := fmt.Sprintf("%s://%s/", u.Scheme, u.Host)
+	pathSegments := strings.Split(strings.Trim(u.Path, "/"), "/")
+	if len(pathSegments) != 2 {
+		f.Logger.Debug("invalid URL path: %s", u.Path)
+		return "", "", "", fmt.Errorf("invalid URL path, ensure url has a structure of /container/filename.ign: %s", u.Path)
+	}
+	container := pathSegments[0]
+	file := pathSegments[1]
+
+	return storageAccount, container, file, nil
+}
+
+func (f *Fetcher) fetchFromAzureBlob(u url.URL, dest io.Writer, opts FetchOptions) error {
+	// Create a context
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
+	storageAccount, container, file, err := f.parseAzureStorageUrl(u)
+	if err != nil {
+		return err
+	}
+
+	// Create Azure Blob Storage client
+	storageClient, err := azblob.NewClient(storageAccount, f.AzSession, nil)
+	if err != nil {
+		f.Logger.Debug("failed to create azblob client: %v", err)
+		return fmt.Errorf("failed to create azblob client: %w", err)
+	}
+
+	downloadStream, err := storageClient.DownloadStream(ctx, container, file, nil)
+	if err != nil {
+		return fmt.Errorf("failed to download blob from container '%s', file '%s': %w", container, file, err)
+	}
+	defer downloadStream.Body.Close()
+
+	// Process the downloaded blob
+	err = f.decompressCopyHashAndVerify(dest, downloadStream.Body, opts)
+	if err != nil {
+		f.Logger.Debug("Error processing downloaded blob: %v", err)
+		return fmt.Errorf("failed to process downloaded blob: %w", err)
+	}
+
+	return nil
+}
+
 // uncompress will wrap the given io.Reader in a decompresser specified in the
 // FetchOptions, and return an io.ReadCloser with the decompressed data stream.
 func (f *Fetcher) uncompress(r io.Reader, opts FetchOptions) (io.ReadCloser, error) {
